﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Business.Test.Mock;
using Common;
using Business.Model;

namespace Business.Test
{
    [TestClass]
    public class TestBase
    {
        protected MockDbAccessor dbAccessor;

        protected MockSourceReader sourceReader;

        protected MockTaskSystem taskSystem;

        protected MockOrderSystem orderSystem;

        [TestInitialize]
        public void TestInitialize()
        {
            this.dbAccessor = new MockDbAccessor();
            this.sourceReader = new MockSourceReader();
            this.taskSystem = new MockTaskSystem();
            this.orderSystem = new MockOrderSystem();
        }

        protected void InitializeSourceReader()
        {
            int readerDataCount = RandomValueGenerator.GenerateRandomNumber(0, 20);
            List<ReaderData> readerData = GetReaderData(readerDataCount);
            sourceReader.InitializeReaderData(readerData);
        }

        private static List<ReaderData> GetReaderData(int readerDataCount)
        {
            var result = new List<ReaderData>();

            for (int i = 0; i < readerDataCount; i++)
            {
                result.Add(new ReaderData()
                {
                    Id = Guid.NewGuid(),
                    Address = RandomValueGenerator.GenerateRandomString(20, true, true, true, false)
                });
            }

            return result;
        }

        protected void InitializeTaskSystem()
        {
            InitializeSourceReader();
            IEnumerable<ReaderData> readerData = sourceReader.GetReaderData();
            int remainAmount = RandomValueGenerator.GenerateRandomNumber(0, readerData.Count());
            IEnumerable<ReaderData> createdTaskReaders = readerData.Take(readerData.Count() - remainAmount);
            sourceReader.InitializeReaderData(createdTaskReaders.ToList());
            new ConcreteTaskCreator(sourceReader, taskSystem, dbAccessor).CreateTask();

            sourceReader.InitializeReaderData(readerData.ToList());
        }

        protected void InitializeOrderSystem()
        {
            InitializeSourceReader();
            new ConcreteTaskCreator(sourceReader, taskSystem, dbAccessor).CreateTask();
            new ConcreateOrderCreator(sourceReader, taskSystem, orderSystem, dbAccessor).CreateOrder();
        }
    }
}
