﻿// -----------------------------------------------------------------------
// <copyright file="ConcreateOrderCreator.cs" company="">
// TODO: Update copyright text.
// </copyright>
// -----------------------------------------------------------------------

namespace Business
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Business.Facade;
    using Business.Model;

    /// <summary>
    /// TODO: Update summary.
    /// </summary>
    public class ConcreateOrderCreator : IOrderCreator
    {
        private readonly IDbAccessor dbAccessor;

        private readonly ITaskSystem taskSystem;

        private readonly IOrderSystem orderSystem;

        private readonly ISourceReader sourceReader;

        public ConcreateOrderCreator(ISourceReader sourceReader, ITaskSystem taskSystem, IOrderSystem orderSystem, IDbAccessor dbAccessor)
        {
            this.sourceReader = sourceReader;
            this.taskSystem = taskSystem;
            this.orderSystem = orderSystem;
            this.dbAccessor = dbAccessor;
        }

        public void CreateOrder()
        {
            ProcessDeletedTasksAndNoNeedOrderTasks();

            ProcessNeedCreateOrderTasks();
        }

        private void ProcessNeedCreateOrderTasks()
        {
            IEnumerable<Task> orderTasks = taskSystem.GetNeedCreateOrderTasks();

            foreach (var task in orderTasks)
            {
                LogRecord record = dbAccessor.GetLogByTaskId(task.Id);

                if (record == null)
                {
                    record = CreateNewLogRecord(task);
                }

                Guid orderId = CreateOrder(record);

                if (orderId != Guid.Empty)
                {
                    UpdateOrderIdToTaskSystem(task, orderId);

                    UpdateOrderIdToDb(record, orderId);
                }
            }
        }

        private void UpdateOrderIdToTaskSystem(Task task, Guid orderId)
        {
            task.OrderId = orderId;
            taskSystem.UpdateTask(task);
        }

        private void UpdateOrderIdToDb(LogRecord record, Guid orderId)
        {
            record.OrderId = orderId;
            record.OrderCreated = DateTime.Now;
            record.OrderStatus = OrderStatus.New;

            dbAccessor.UpdateLog(record);
        }

        private Guid CreateOrder(LogRecord record)
        {
            Order newOrder = new Order();
            newOrder.ReaderId = record.ReaderId;
            newOrder.Address = record.Address;

            return orderSystem.CreateOrder(newOrder);
        }

        private LogRecord CreateNewLogRecord(Task task)
        {
            ReaderData readerItem = sourceReader.GetReaderData().SingleOrDefault(r => r.Id == task.ReaderId);

            if (readerItem != null)
            {
                LogRecord record = Utility.GetNewLogRecord(task, readerItem);

                dbAccessor.CreateLog(record);

                return record;
            }

            throw new ArgumentException("Wrong reader");
        }

        private void ProcessDeletedTasksAndNoNeedOrderTasks()
        {
            IEnumerable<LogRecord> uncompleteRecords = dbAccessor.GetUncompleteRecords();

            IEnumerable<Task> tasks = taskSystem.GetTasks(uncompleteRecords.Select(r => r.TaskId).ToArray());

            foreach (var record in uncompleteRecords)
            {
                var task = tasks.SingleOrDefault(t => t.Id == record.TaskId);

                if (task == null)
                {
                    dbAccessor.DeleteLog(record.Id);
                }
                else
                {
                    if (!task.NeedCreateOrder && task.Status == TaskStatus.Completed && !record.IsCompleted)
                    {
                        record.IsCompleted = true;

                        dbAccessor.UpdateLog(record);
                    }
                }
            }
        }
    }
}
